/**********************************************************************
 * $Id: example.java,v 1.12 2006/10/03 07:05:18 nacho Exp $
 *
 * Name:     example.java
 * Project:  JGDAL. Interface java to gdal (Frank Warmerdam).
 * Purpose:  Example code to test java interface of gdal. 
 * Author:   Nacho Brodin, brodin_ign@gva.es
 *
 **********************************************************************/
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/



/**
 * Sections that correspond with GDAL Api Tutorial (publicated in
 * http://www.remotesensing.org:16080/gdal/gdal_tutorial.html) have
 * been marked.
 */

//Opening the file ----------------------------

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import es.gva.cit.jgdal.Gdal;
import es.gva.cit.jgdal.GdalBuffer;
import es.gva.cit.jgdal.GdalColorEntry;
import es.gva.cit.jgdal.GdalColorTable;
import es.gva.cit.jgdal.GdalException;
import es.gva.cit.jgdal.GdalRasterBand;
import es.gva.cit.jgdal.GeoTransform;

public class example{
	public static void main(String[] args){
 
	  
	  Gdal migdal=new Gdal();
	  
	  try{
	    if(args.length>0)
	    	migdal.open(args[0],Gdal.GA_ReadOnly);
	    else {
	    	System.out.println("You must introduce the file name"); 
	    	System.exit(1);
	    }
	  }catch(Exception ge){
			ge.printStackTrace();
			//...........
	  }

//End Opening the file ----------------------------
	  
	  
	  GdalRasterBand mirasterband=null;
	  GdalRasterBand mioverview=null;
  
	  FileOutputStream s=null;
	  DataOutputStream midataOS=null;
	  BufferedOutputStream miBS=null;
	  String st;
	  int nxsize=0;
	  int nysize=0;
	  int rastercount=0;
	  String rasterband;
	  int xoverview=0;
	  int yoverview=0;
	  GeoTransform gt;
	  int noverviews=0;
	    
//Getting dataset Information ----------------------------	  
	  
	  try{
	  	rastercount=migdal.getRasterCount();
	    nxsize = migdal.getRasterXSize();
	    nysize = migdal.getRasterYSize();
	    System.out.println("N BANDAS="+rastercount+" SIZEX="+nxsize+" SIZEY="+nysize);
	    System.out.println(migdal.getProjectionRef());	
	  }catch(GdalException ge){
	  						ge.printStackTrace();
	  						//...........
	  					   }
	  
	  
	  
	  try{
	  	gt=migdal.getGeoTransform();
	  	System.out.println("Origin = ("+gt.adfgeotransform[0]+","+gt.adfgeotransform[3]+")");
	  	System.out.println("Pixel Size = ("+gt.adfgeotransform[1]+","+gt.adfgeotransform[5]+")");
	  }catch(GdalException e){
	  		System.out.println("I can't obtain the array geoTransform for this image");
			e.printStackTrace();
			//...........
	  }
	  
//End Getting dataset Information ----------------------------	  
	  
	  try{
	       File mifichero = new File("exit.raw");
	       s = new FileOutputStream(mifichero);
	  }catch(Exception e){}
	  

	  for(int iBand=0;iBand<rastercount;iBand++){
	  	
	  	System.out.println("-------");
	  	
	  	//Informacion de banda

	  
//Fetching a Raster Band ----------------------------	  
//Several binding are missing yet	  	
	  	
	  	int tipo=0;
	  	try{
	  		mirasterband = migdal.getRasterBand(iBand+1);
	  		System.out.println("Band Size: ("+mirasterband.getRasterBandXSize()+","+mirasterband.getRasterBandYSize()+")");
	  		System.out.println("Block Size: ("+mirasterband.getBlockXSize()+","+mirasterband.getBlockYSize()+")");
	  		noverviews = mirasterband.getOverviewCount();
	  		System.out.println("Number of Overview: "+noverviews);
	  		tipo = mirasterband.getRasterDataType();
	  		System.out.println("Band data type: "+tipo);
	  		double noData = mirasterband.getRasterNoDataValue();
	  		if(noData >= 0)
	  			System.out.println("NoData="+noData);
	  		String[] metadata = mirasterband.getMetadata();
		    if(metadata.length > 0){
		    	System.out.println("-METADATA-");
		    	for(int i=0;i<metadata.length;i++)
		    		System.out.println(metadata[i]);
		    }
		    
		    //Palette
		    String colorInt = migdal.getColorInterpretationName(mirasterband.getRasterColorInterpretation());
		    System.out.println("ColorInterpretation: "+colorInt);
		    if(colorInt.equals("Palette")){
		    	GdalColorTable table = mirasterband.getRasterColorTable();
		    	System.out.println("Number of entries: "+table.getColorEntryCount());
		    	for(int nEntry = 0; nEntry < table.getColorEntryCount(); nEntry++ ){
		                GdalColorEntry	sEntry;
		                sEntry = table.getColorEntryAsRGB(nEntry);
		                System.out.println( 
		                		nEntry + " (" +
		                        sEntry.c1 + ", " +
		                        sEntry.c2 + ", " +
		                        sEntry.c3 + ", " +
		                        sEntry.c4  + ")");
		        }

		    }
		    
	  	}catch(GdalException e){
	  		e.printStackTrace();
	  		System.exit(1);
	  	}
	  	
//Fetching a Raster Band ----------------------------

	  	System.out.println("BANDA->"+iBand);
	  	
	  	//Informacin de overview
	  	
	  	try{
	  			  		
	  		if(noverviews>0){
	  			
	  			//Muestra el tamao de cada overview y selecccionamos el 0
	  			
	  			for(int t=0;t<noverviews;t++){
	  				mioverview=mirasterband.getOverview(t);
	  				System.out.println("SIZE OVERVIEW=("+mioverview.getRasterBandXSize()+","+mioverview.getRasterBandYSize()+")");
	  			}
	  			mioverview=mirasterband.getOverview(0);
	  			xoverview=mioverview.getRasterBandXSize();
	  			yoverview=mioverview.getRasterBandYSize();
	  		}else{
	  			System.out.println("This band haven't overviews");
		  		xoverview=nxsize;
				yoverview=nysize;
				mioverview=mirasterband;	
	  			
	  		}
	  		
	  	}catch(GdalException e){
	  		e.printStackTrace();
	  		System.exit(1);
	  	}
	  	
	  	
	  	
	  	
	  	

	  	
		//byte[] buf=null;
		GdalBuffer buf=null;
		
	  	for(int i=0; i<yoverview; i++){

//Reading Raster Data ----------------------------	  
	  		
		  	try{
		  		buf = mioverview.readRaster(0, i, xoverview, 1, xoverview, 1, tipo);
		  	}catch(GdalException e){
		  		System.out.println("Error in BSBReadLine");
		  		System.exit(1);
		  	}

//End Reading Raster Data ----------------------------		  	
		  	
	  		try{
	  			if(tipo==Gdal.GDT_Byte)s.write(buf.buffByte);
	  		}catch(IOException e){
	  			System.out.println("Error writing a line in the file\n");	
	  			System.exit(1);
	  		}
		
	  	}
	  	
	}
	  
	 
	  
	  try{	  	
	    String[] metadata = migdal.getMetadata();
	    if(metadata.length > 0){
	    	System.out.println("-METADATA-");
	    	System.out.println("Number of values:"+metadata.length);
	    	for(int i=0;i<metadata.length;i++)
	    		System.out.println(metadata[i]);
	    }
	    migdal.close();
	  }catch(GdalException e){
	  							e.printStackTrace();
	  						 }
	  
	  
	}  
}